// --------------------------------------------------------------
//  T6963C_B0_lib
// --------------------------------------------------------------
//  Rutinas de BAJO nivel para GLCD
// --------------------------------------------------------------
//
//          sget(): lee el "status"
//          cput(): escribe un byte de control
//          dput(): escribe un byte de datos
//          dget(): lee un byte de datos
//          espera(): espera el "status = Ok"
//
// --------------------------------------------------------------
#include "T6963C_B0.h"
// --------------------------------------------------------------
/*
 * La placa usa el controlador T6963C
 * --------------------------------------------------------------
 *  FG    (1)  frame ground
 *  GND   (2) <--> (25) GND
 *  +5V   (3)  LCD logic supply
 *  -7.8V (4)  LCD contrast
 * --------------------------------------------------------------
 *  LCD Pin ----- PC Port Pin  Status Reg. bit     PIC
 *  /WR  (5) <--> (16) Init      2                  4
 *  /RD  (6) <--> (14) /LF       1                  5
 *  /CE  (7) <--> (1)  /Strobe   0                  3
 *  C/D  (8) <--> (17) /SEL      3                  0
 * --------------------------------------------------------------
 * LCD pines para WM-G1212A - 128x128 pixels
 *
 * _RST  (9) 0:reset 1:normal op.
 *  FS  (18) font select
 *  LCD Pin ----- PORTB pin
 *  D0  (10) <--> RB0 (0)
 *  D1  (11) <--> RB1
 *  D2  (12) <--> RB2
 *  D3  (13) <--> RB3
 *  D4  (14) <--> RB4
 *  D5  (15) <--> RB5
 *  D6  (16) <--> RB6
 *  D7  (17) <--> RB7 (7)
 * --------------------------------------------------------------
 * LCD pines para WM-G12864 - 128x64 pixels
 *
 * _RST (--) 0:reset 1:normal op.
 *  FS  (--) font select
 *  LCD Pin ----- PORTB pin
 *  D0  (--) <--> RB0 (0)
 *  D1  (--) <--> RB1
 *  D2  (--) <--> RB2
 *  D3  (--) <--> RB3
 *  D4  (--) <--> RB4
 *  D5  (--) <--> RB5
 *  D6  (--) <--> RB6
 *  D7  (--) <--> RB7 (7)
 * --------------------------------------------------------------
 * LCD pines para TOSHIBA TLX-711A-E0 - 240x64 pixels
 *
 * _RST (10) 0:reset 1:normal op.
 *  FS  (19) font select
 *  LCD Pin ----- PC Port Pin
 *  D0  (11) <--> RB0 (0)
 *  D1  (12) <--> RB1
 *  D2  (13) <--> RB2
 *  D3  (14) <--> RB3
 *  D4  (15) <--> RB4
 *  D5  (16) <--> RB5
 *  D6  (17) <--> RB6
 *  D7  (18) <--> RB7 (7)
 */
// --------------------------------------------------------------
void espera_() // Para etapa de debug
{
//while ((0x03 & sget()) != 0x03) continue;
}
// --------------------------------------------------------------
void espera()
{
  unsigned int nst;

  nst=10;
  while (nst>0 && ( 0x03 & sget() != 0x03)) {nst--;}

//  nst=100;
//  while (nst>0 && ( 0x80 & sget() != 0x80)) {nst=nst-1;}
}
// ----------------------------------------------------------------
unsigned short sget(void)  // get LCD display status byte
{
  unsigned short lcd_status;

  TRISB=0xFF; //  Deja el puerto como ENTRADA

  PORTC.F3=1; // CEHI;   // desactiva chip
  PORTC.F0=1; // CDHI;   // modo status
  PORTC.F4=1; // WRHI;   // desactiva grabacion
  PORTC.F5=0; // RDLO;	 // activa lectura

  PORTC.F3=0; // CELO;	 // activa chip
  PORTC.F3=0; // CELO;   // repite para superar los 80 ns
  lcd_status=PORTB; // lee el puerto

  PORTC.F3=1; // CEHI;   // desactiva chip
  PORTC.F5=1; // RDHI;   // desactiva lectura

  return(lcd_status);
} // sget()
// ----------------------------------------------------------------
unsigned short dget(void)      // get data byte from LCD module
{
  unsigned short lcd_byte;

  espera();   // Deja el puerto como ENTRADA

  PORTC.F3=1; // CEHI;   // desactiva chip
  PORTC.F0=0; // CDLO;   // modo dato
  PORTC.F4=1; // WRHI;   // desactiva grabacion
  PORTC.F5=0; // RDLO;	 // activa lectura

  PORTC.F3=0; // CELO;	 // activa chip
  PORTC.F3=0; // CELO;   // repite para superar los 80 ns
  PORTC.F3=0; // CELO;   // repite para superar los 150 ns
  lcd_byte=PORTB; // lee el puerto

  PORTC.F3=1; // CEHI;   // desactiva chip
  PORTC.F5=1; // RDHI;   // desactiva lectura

  return(lcd_byte);
} // dget()
// ----------------------------------------------------------------
void dput(unsigned short byte) // write data byte to LCD module
{
  espera();   // Deja el puerto como ENTRADA

  TRISB=0;    //  Deja el puerto como SALIDA

  PORTC.F3=1; // CEHI;   // desactiva chip
  PORTC.F0=0; // CDLO;   // modo dato
  PORTC.F5=1; // RDHI;	 // desactiva lectura
  PORTC.F4=0; // WRLO;   // activa grabacion

  PORTB=byte; // escribe en el puerto
  PORTC.F3=0; // CELO;	 // activa chip
  PORTC.F3=0; // CELO;   // repite para superar los 80 ns

  PORTC.F3=1; // CEHI;   // desactiva chip
  PORTC.F4=1; // WRHI;   // desactiva grabacion

  TRISB=0xFF; //  Deja el puerto como ENTRADA

} // end dput()
// ----------------------------------------------------------------
void cput(unsigned short byte) // write command byte to LCD module
					// assumes port is in data OUTPUT mode
{
  espera();   // Deja el puerto como ENTRADA

  TRISB=0;    //  Deja el puerto como SALIDA

  PORTC.F3=1; // CEHI;   // desactiva chip
  PORTC.F0=1; // CDHI;   // modo comando
  PORTC.F5=1; // RDHI;	 // desactiva lectura
  PORTC.F4=0; // WRLO;   // activa grabacion

  PORTB=byte; // escribe en el puerto
  PORTC.F3=0; // CELO;	 // activa chip
  PORTC.F3=0; // CELO;   // repite para superar los 80 ns

  PORTC.F3=1; // CEHI;   // desactiva chip
  PORTC.F4=1; // WRHI;   // desactiva grabacion

  TRISB=0xFF; //  Deja el puerto como ENTRADA

  } // cput()
// --------------------------------------------------------------

