// ================================================================
// T6963C_B1.h
// ================================================================
// Rutinas de alto nivel, para grficas y texto
// --------------------------------------------------------------
#include "T6963C_B0_lib.c"
#include "T6963C_B1.h"
//--------------------------------------------------------------
unsigned short _NRO_DE_COLUMNAS_GR;
unsigned short _NRO_DE_COLUMNAS_TX;
unsigned short _NRO_DE_FILAS;
unsigned short _MODO_DISPLAY;      // Activacion: MGRAF+MTEXT+CURS+BLINK
unsigned short _H_PIXEL;           // Ancho del caracter (6 / 8)
// ================================================================
void glcd_config(unsigned short x_Size, unsigned short y_Size,
     unsigned short Ancho_byte_TX, unsigned short Ancho_byte_GR)
{
	_H_PIXEL = Ancho_byte_GR;
	_NRO_DE_COLUMNAS_GR = (x_Size+3)/Ancho_byte_GR;
	_NRO_DE_COLUMNAS_TX = (x_Size+3)/Ancho_byte_TX;
	_NRO_DE_FILAS       = (y_Size+3)/8;
  //
  TRISB = 0xFF;       // ENTRADA por defecto
  TRISC = 0x86;       // Para mediciones
  PORTC = 0xFF;       // Deshablita todo el display
  //
} // glcd_config
// --------------------------------------------------------------
// initialize LCD memory and display modes
void glcd_inicio()
  {
  dput(G_BASE%256);	// Pone Byte bajo
	dput(G_BASE>>8);	// Pone Byte alto
	cput(0x42);     	// set graphics memory to address G_BASE

	dput(_NRO_DE_COLUMNAS_GR%256);// Pone Byte bajo
	dput(_NRO_DE_COLUMNAS_GR>>8); // Pone Byte alto
	cput(0x43);  	 	           // n bytes per graphics line

	dput(T_BASE%256);	         // Pone Byte bajo
	dput(T_BASE>>8); 	         // Pone Byte alto
	cput(0x40);     	         // text memory at address T_BASE

	dput(_NRO_DE_COLUMNAS_TX%256);// Pone Byte bajo
	dput(_NRO_DE_COLUMNAS_TX>>8); // Pone Byte alto
	cput(0x41);     	         // n bytes per text line

	cput(0x80);     	         // mode set: Graphics OR Text, ROM CGen

  _MODO_DISPLAY = 0x90;      // Graphics, Text, cursor & blinking = OFF
	cput(_MODO_DISPLAY);

//  _x_cur = 0;    _y_cur = 0;
  dput(1);dput(1);cput(0x21);// put cursor at (1,1)

//  _altocur = 1;              // 2 pixels de alto
	cput(0xa1);	                 // cursor is 2 lines high

} // glcd_inicio
// ================================================================
//  Para memoria
// --------------------------------------------------------------
unsigned int calcular_pos_mem(unsigned int BASE, unsigned int LARGO,
         unsigned short FILA)
{
unsigned int addr;
unsigned short valor;

addr  = BASE;
valor = 0;
while ( valor<FILA )
    {
    valor= valor+ 1;
    addr = addr + LARGO;
    }
return (addr);
} // calcula_pos_mem
// ----------------------------------------------------------------
// Llena la memoria del mismo caracter
// Sirve para limpiar textos o graficos
// o para poner un papel tapiz grafico
void glcd_fill_byte(unsigned short COLUMNAS, unsigned short FILAS,
     unsigned char ch)
{
unsigned short i,j;
for (j=0;j<FILAS;j++)
    {
    for (i=0;i<COLUMNAS;i++)
		 {
     dput(ch);cput(0xc0);   	// write data, inc ptr.
     } // end for(i)
	}// end for(j)
} // end glcd_fill_byte
// ================================================================
//  Para puntero de memoria
// --------------------------------------------------------------
// set memory pointer to graf/text position (x,y)
unsigned int glcd_put_ptr(unsigned int BASE,  unsigned int LARGO,
     unsigned short COLUMNA, unsigned short FILA)
{
unsigned int addr;
addr = calcular_pos_mem( BASE + COLUMNA, LARGO, FILA );
dput(addr%256); dput(addr>>8); cput(0x24);  // set LCD addr. pointer
return(addr);
} // glcd_put_ptr
// ================================================================
//  Para modo
// --------------------------------------------------------------
void glcd_activar(unsigned short MASK)
{
_MODO_DISPLAY = _MODO_DISPLAY | MASK;
cput(_MODO_DISPLAY | MASK);// Prende
}
//
void glcd_desactivar(unsigned short MASK)
{
_MODO_DISPLAY = _MODO_DISPLAY & ~MASK;
cput(_MODO_DISPLAY & ~MASK); // Apaga
}
// ================================================================
//  Para cursor
// --------------------------------------------------------------
void glcd_set_cursor_height(unsigned short H)
{
//unsigned char _altocur;
//_altocur = H;              // H-1 pixels de alto
cput(0xa0 + (char)H);        // cursor is altocur+1 lines high
}
// ----------------------------------------------------------------
void glcd_put_cursor(unsigned short XC, unsigned short YC)
{
dput(XC);dput(YC);cput(0x21);// put cursor at (XC,YC)
}
// ================================================================
//  Funciones del modo grfico
// ----------------------------------------------------------------
// set or clr a single pixel in 128x128 array
void glcd_paint_pixel(unsigned short COLUMNA, unsigned short FILA,
     unsigned short color)
{
unsigned int addr;       // memory address of byte containing pixel to write
unsigned short valor;
unsigned int bufer;
unsigned short resto;

valor = COLUMNA;
bufer = 0;
while( valor - _H_PIXEL >= 0 )
    {
    valor = valor - _H_PIXEL;
    bufer = bufer + 1;
    }
resto = valor ;   //bufer = column/6;
addr = calcular_pos_mem( G_BASE + bufer,_NRO_DE_COLUMNAS_GR,FILA );
dput(addr%256);dput(addr>>8);cput(0x24);			// set LCD addr. pointer
resto = _H_PIXEL - 1 - resto; //resto = 5 - column% _H_PIXEL;
if (color==1)
    cput( 0xf8 |  resto );	// set bit-within-byte command
else
    cput( 0xf8 & ~resto );	// clr bit-within-byte command
} // lcd_paint_pixel
// ----------------------------------------------------------------
void glcd_horizontal( char y_pix,char x_min,char x_max,char color)
{
char x_pix;
for(x_pix=x_min;x_pix<=x_max;x_pix++)
       glcd_paint_pixel(x_pix,y_pix,color);
}
// ----------------------------------------------------------------
void glcd_vertical(char x_pix,char y_min,char y_max,char color)
{
char y_pix;
if (color>1)color=1;
for(y_pix=y_min;y_pix<=y_max;y_pix++)
       glcd_paint_pixel(x_pix,y_pix,color);
}
// ----------------------------------------------------------------
void glcd_draw_rectangle( unsigned short x0, unsigned short y0,
     unsigned short xf, unsigned short yf, unsigned short color)
{
glcd_horizontal(x0,xf,y0,color);
glcd_vertical(y0,yf,x0,color);
glcd_horizontal(x0,xf,yf,color);
glcd_vertical(y0,yf,xf,color);
}
// ----------------------------------------------------------------
void glcd_draw_ellipse(int x0,int y0,int rx,int ry,int color)
{
int n,Np, x,y;
double ang,dang;

Np = 6. * sqrt( 1. * (rx*rx+ ry*ry) );

ang = 0;
dang = 2. * PI / Np;
for(n=0;n<Np;n++)
	{
	x = x0 + rx * cos(ang);
	y = y0 + ry * sin(ang);
  glcd_paint_pixel(x,y,color);
	ang += dang;
	}
}
// ----------------------------------------------------------------
void glcd_draw_sprite(unsigned short x0, unsigned short y0,
     unsigned short NCOLS, unsigned short NFILS,
     const char *array )
{
    unsigned short i, j;
//    unsigned int y2;

//    y2 = 0;
    for (j=0;j<NFILS;j++)
        {
        glcd_put_ptr(G_BASE,_NRO_DE_COLUMNAS_GR, x0, y0 + j);
        for (i=0;i<NCOLS;i++)
            {
            //dput(array[y2+i]);cput(0xc0);
            dput(array[j*NCOLS+i]);cput(0xc0);
            }
//        y2 = y2 + NCOLS;
        }
}
// ----------------------------------------------------------------
void glcd_draw_fondo(unsigned short x0, unsigned short y0,
     unsigned short NCOLS, unsigned short NFILS,
     unsigned char caracter )
{
    unsigned short i, j;
//    unsigned int y2;

//    y2 = 0;
    for (j=0;j<NFILS;j++)
        {
        glcd_put_ptr(G_BASE,_NRO_DE_COLUMNAS_GR, x0, y0 + j);
        for (i=0;i<NCOLS;i++)
            {
            dput(caracter);cput(0xc0);
            }
//        y2 = y2 + NCOLS;
        }
}
// ================================================================
//  Funciones del modo texto
// ----------------------------------------------------------------
void glcd_putchar(char *letra)  // send char to LCD
{
int c;
	c = *letra - 0x20;  // convert ASCII to LCD char address
	if (c<0) c=0; // Si c==0 manda Blanco
	dput(c);      // Manda Dato
	cput(0xc0);   // Manda Comando: Enva el caracter, mantiene puntero
}
// ----------------------------------------------------------------
void glcd_print(char *string)  // send string of characters to LCD
{
unsigned short i;
int largo;

largo = strlen(string);
if (largo > _NRO_DE_COLUMNAS_TX)
   {
   largo = _NRO_DE_COLUMNAS_TX;
	 } // end for

for (i=0;i<largo;i++)
   {
	 glcd_putchar(&string[i]);
	 } // end for

} // end lcd_print
// ----------------------------------------------------------------
void Mostrar_Texto( unsigned short x, unsigned short y,
     unsigned char *string)
{
glcd_put_ptr(T_BASE,_NRO_DE_COLUMNAS_TX,x,y);
glcd_print(string);
}
// ----------------------------------------------------------------
void Limpiar_Linea( unsigned short y )
{
glcd_put_ptr(T_BASE,_NRO_DE_COLUMNAS_TX,0,y);
glcd_fill_byte(22,1,0);
}
// ================================================================
